/******************************************************************
Copyright (c) 1996 Novell, Inc. All Rights Reserved.

THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND
TREATIES. USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE
LICENSE AGREEMENT ACCOMPANYING THE SOFTWARE DEVELOPMENT KIT (SDK)
THAT CONTAINS THIS WORK. PURSUANT TO THE SDK LICENSE AGREEMENT,
NOVELL HEREBY GRANTS TO DEVELOPER A ROYALTY-FREE, NON-EXCLUSIVE
LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS PRODUCT. NOVELL
GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF
DEVELOPER'S PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO
DEVELOPER OR DEVELOPER'S CUSTOMERS WITH RESPECT TO THIS CODE.
******************************************************************/

package com.novell.nds.dirxml.driver.jdbc.db;


/**
 * An enumeration of directory time formats.
 */
public
class DSTimeFormat
{

    private static final int NUM_FORMATS = 3;

    public static final DSTimeFormat INT        = new DSTimeFormat(1);
    public static final DSTimeFormat STRING     = new DSTimeFormat(2);
    public static final DSTimeFormat JAVA       = new DSTimeFormat(3); //no change
    public static final DSTimeFormat DEFAULT    = INT;

    private static final DSTimeFormat[] FORMATS;

    static
    {
        FORMATS = new DSTimeFormat[NUM_FORMATS];
        FORMATS[0] = INT;
        FORMATS[1] = STRING;
        FORMATS[2] = JAVA;
    }


    /**
     * Translates a <code>String</code> into its corresponding
     * <code>DSTimeFormat</code> representation.
     *
     * @param someSyntax may be <code>null</code>
     * @return may return <code>null</code>
     */
    static public DSTimeFormat GetFormat(String someSyntax)
    {
        for(int s=0; s<FORMATS.length; s++)
        {
            if(FORMATS[s].equals(someSyntax))
            {
                return FORMATS[s];
            }
        }//for

        return null;
    }//GetFormat()


    private String string;


    private DSTimeFormat(int syntax)
    {
        this.string = String.valueOf(syntax);
    }


    /**
     * Returns the <code>String</code> representation of this instance.
     *
     * @return will not return <code>null</code>
     */
    public String toString()
    {
        return this.string;
    }


    /**
     * Returns the hashcode of this instance.
     *
     * @return an integer value
     */
    public int hashCode()
    {
        return this.string.hashCode();
    }


    /**
     * Is this format equal to the passed <code>String</code> representation?
     *
     * @return <code>true</code> if equal; <code>false</code>
     * otherwise
     */
    public boolean equals(String value)
    {
        return this.string.equalsIgnoreCase(value);
    }

}//class DSTimeFormat
