<?xml version='1.0'?>
<!--
*********************************************************************** 
* Name:			upgraderules.xsl                                         
* Description:	Stylesheet for upgrading rules from previous versions    
*                 of DirXML                                                
*                                                                          
*  Copyright (C) 2003 Novell, Inc., All Rights Reserved                    
*                                                                          
*  %version:			9 %                                                  
*  %created_by:  	svella %                                                 
*  %date_modified:	Wed Apr 05 10:31:45 2006 %                           
***********************************************************************
-->

<xsl:transform
		version="1.0"
       xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	   >

<!-- This is for testing the stylesheet outside of DirXML so things are pretty to look at -->
<xsl:strip-space elements="*"/>
<xsl:preserve-space elements="association add-association remove-association value component password check-password xsl:text token-text placement"/>
<xsl:output indent="yes" method="xml"/>

       
<!-- convert the old 1.x create rules to policies -->
<xsl:template match="create-rules">
	<policy>
		<xsl:apply-templates select="create-rule"/>
	</policy>	
</xsl:template>

<!-- convert the old 1.x placement rules to policies -->
<xsl:template match="placement-rules">
	<policy>
		<xsl:apply-templates select="placement-rule"/>
	</policy>	
</xsl:template>

<!-- convert the old 1.x matching rules to policies -->
<xsl:template match="matching-rules">
	<policy>
		<xsl:apply-templates select="matching-rule"/>
	</policy>	
</xsl:template>

<!-- convert a create rule -->
<xsl:template match="create-rules/create-rule">
	<rule>
		<xsl:apply-templates select="@description"/>
		<conditions>
			<or>
				<xsl:apply-templates select="@class-name"/>
			</or>
			<xsl:for-each select="match-attr/value">
				<or>
					<xsl:apply-templates select="."/>
				</or>
			</xsl:for-each>
		</conditions>
		<actions>
			<xsl:apply-templates select="required-attr[not(value)]"/>
			<xsl:apply-templates select="required-attr[value]"/>
			<xsl:apply-templates select="template"/>
			<do-break/>
		</actions>
	</rule>
</xsl:template>

<!-- convert a rule description -->
<xsl:template match="create-rule/@description | placement-rule/@description | matching-rule/@description">
	<xsl:call-template name="mimic-indent"/>
	<description>
		<xsl:value-of select="."/>
	</description>
</xsl:template>

<!-- convert create rule class-name -->
<xsl:template match="create-rule/@class-name">
	<xsl:call-template name="mimic-indent"/>
	<if-class-name op="equal">
		<xsl:value-of select="."/>
	</if-class-name>
</xsl:template>

<!-- make a half hearted attempt to mimic the surrounding indentation -->
<xsl:template name="mimic-indent">
	<xsl:copy-of select="../text()[1]"/>
</xsl:template>

<!-- convert a placement and create rule match-attr -->
<xsl:template match="match-attr/value">
	<if-op-attr op="equal" name="{../@attr-name}">
		<xsl:choose>
			<xsl:when test="component or @type='structured'">
				<xsl:attribute name="mode">structured</xsl:attribute>
			</xsl:when>
			<xsl:otherwise>
				<!-- calling code needs to examine the schema and determine what the mode should be -->
				<xsl:attribute name="mode">nocase</xsl:attribute>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:apply-templates />
	</if-op-attr>
</xsl:template>

<!-- convert create rule required-attr without any default values -->
<xsl:template match="required-attr[not(value)]">
	<do-veto-if-op-attr-not-available name="{@attr-name}"/>
</xsl:template>

<!-- convert create rule required-attr with default values -->
<xsl:template match="required-attr[value]">
	<xsl:apply-templates select="value"/>
</xsl:template>

<!-- convert create rule required-attr default values -->
<xsl:template match="required-attr/value">
	<do-set-default-attr-value name="{../@attr-name}">
		<xsl:apply-templates select="@write-back" />
		<arg-value>
			<xsl:choose>
				<xsl:when test="component or @type='structured'">
					<xsl:attribute name="type">structured</xsl:attribute>
					<xsl:apply-templates/>
				</xsl:when>
				<xsl:otherwise>
					<token-text>
						<xsl:attribute name="xml:space">preserve</xsl:attribute>
						<xsl:apply-templates/>
					</token-text>
				</xsl:otherwise>
			</xsl:choose>
		</arg-value>
	</do-set-default-attr-value>
</xsl:template>

<!-- convert create rule required-attr default value components -->
<xsl:template match="required-attr/value/component">
	<arg-component>
		<xsl:apply-templates select="@name"/>
		<token-text>
			<xsl:attribute name="xml:space">preserve</xsl:attribute>
			<xsl:apply-templates/>
		</token-text>
	</arg-component>
</xsl:template>

<!-- convert create rule template -->
<xsl:template match="create-rule/template">
	<do-set-op-template-dn>
		<arg-dn>
			<token-text>
				<xsl:attribute name="xml:space">preserve</xsl:attribute>
				<xsl:value-of select="@template-dn"/>
			</token-text>
		</arg-dn>
	</do-set-op-template-dn>
</xsl:template>

<!-- convert matching rule -->
<xsl:template match="matching-rules/matching-rule">
	<xsl:choose>
		<xsl:when test="match-attr">
			<rule>
				<xsl:apply-templates select="@description"/>
				<conditions>
					<xsl:if test="match-class">
						<or>
							<xsl:apply-templates select="match-class"/>
						</or>
					</xsl:if>
				</conditions>
				<actions>
					<do-find-matching-object scope="subtree">
						<xsl:apply-templates select="match-path"/>
						<xsl:apply-templates select="match-attr"/>
					</do-find-matching-object>
				</actions>
			</rule>
		</xsl:when>
		<xsl:otherwise>
			<!-- if no match-attr, then the rule essentially did nothing, don't bother translating it -->
			<xsl:comment>Omitting rule that had no effect in DirXML 1.1 when converting to DirXMLScript</xsl:comment>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<!-- convert matching and placement rule match-class -->
<xsl:template match="match-class">
	<if-class-name op="equal">
		<xsl:value-of select="@class-name"/>
	</if-class-name>
</xsl:template>

<!-- convert matching rule match-attr -->
<xsl:template match="matching-rule/match-attr">
    <arg-match-attr name="{@attr-name}"/>
</xsl:template>

<!-- convert matching rule match-path -->
<xsl:template match="matching-rule/match-path">
    <arg-dn> 
        <token-text>
			<xsl:attribute name="xml:space">preserve</xsl:attribute>
			<xsl:if test="not(variable-ref/@attr-name[.='prefix'] or request-data/@attr-name[.='prefix'])">
				<xsl:value-of select="@prefix"/>
			</xsl:if>
			<xsl:apply-templates/>
		</token-text>
    </arg-dn> 
</xsl:template>

<!-- convert placement rule -->
<xsl:template match="placement-rules/placement-rule">
	<rule>
		<xsl:apply-templates select="@description"/>
		<conditions>
			<xsl:if test="match-class">
				<or>
					<xsl:apply-templates select="match-class"/>
				</or>
			</xsl:if>
			<xsl:if test="not(match-path) and (placement/copy-path or placement/copy-path-suffix or placement/copy-name)">
				<or>
					<if-src-dn op="available"/>
				</or>
			</xsl:if>
			<xsl:for-each select="match-path | request-data">
				<xsl:apply-templates select="."/>
			</xsl:for-each>
			<xsl:for-each select="match-attr/value">
				<or>
					<xsl:apply-templates select="."/>
				</or>
			</xsl:for-each>
			<xsl:for-each select="placement/copy-attr">
				<or>
					<if-op-attr op="available" name="{@attr-name}"/>
				</or>
			</xsl:for-each>
		</conditions>
		<actions>
			<xsl:apply-templates select="placement"/>
		</actions>
	</rule>
</xsl:template>

<!-- convert placement rule match-path -->
<xsl:template match="placement-rule/match-path">
	<or>
		<if-src-dn op="in-subtree">
			<xsl:attribute name="xml:space">preserve</xsl:attribute>
			<xsl:if test="not(variable-ref/@attr-name[.='prefix'] or request-data/@attr-name[.='prefix'])">
				<xsl:value-of select="@prefix"/>
			</xsl:if>
			<xsl:apply-templates/>
		</if-src-dn>
	</or>
	<or>
		<if-src-dn op="not-equal">
			<xsl:attribute name="xml:space">preserve</xsl:attribute>
			<xsl:if test="not(variable-ref/@attr-name[.='prefix'] or request-data/@attr-name[.='prefix'])">
				<xsl:value-of select="@prefix"/>
			</xsl:if>
			<xsl:apply-templates/>
		</if-src-dn>
	</or>
</xsl:template>

<!-- convert placement rule placement -->
<xsl:template match="placement">
	<do-set-op-dest-dn> 
	    <arg-dn> 
			<xsl:apply-templates/>
	    </arg-dn> 
	</do-set-op-dest-dn>
	<do-break/>
</xsl:template>

<!-- convert placement rule copy-name -->
<xsl:template match="copy-name">
	<token-src-name/>
</xsl:template>

<!-- convert placement rule copy-attr -->
<xsl:template match="copy-attr">
	<token-escape-for-dest-dn>
		<token-op-attr name="{@attr-name}"/>
	</token-escape-for-dest-dn>
</xsl:template>

<!-- convert placement rule copy-path -->
<xsl:template match="copy-path">
	<token-src-dn convert="true"/>
</xsl:template>

<!-- convert placement rule copy-path-suffix -->
<xsl:template match="copy-path-suffix">
	<token-unmatched-src-dn convert="true"/>
</xsl:template>


<!-- convert placement rule literal placement text -->
<xsl:template match="placement/text()">
	<xsl:variable name="normalized" select="translate(.,'&#x0a;&#x09;','')"/>
	<xsl:if test="$normalized != ''">
		<token-text>
			<xsl:attribute name="xml:space">preserve</xsl:attribute>
			<xsl:value-of select="$normalized"/>
		</token-text>
	</xsl:if>
</xsl:template>



<!-- a whole bunch of cases to try to handle variable references and data requests in a preconfigured driver file -->

<xsl:template match="placement/variable-ref |
					placement/request-data">
	<token-text>
		<xsl:attribute name="xml:space">preserve</xsl:attribute>
		<xsl:copy-of select="."/>
	</token-text>
</xsl:template>


<xsl:template match="create-rule//variable-ref/@attr-name[. = 'attr-name'] |
					placement-rule//variable-ref/@attr-name[. = 'attr-name'] |
					matching-rule//variable-ref/@attr-name[. = 'attr-name'] |
					create-rule//request-data/@attr-name[. = 'attr-name'] |
					placement-rule//request-data/@attr-name[. = 'attr-name'] |
					matching-rule//request-data/@attr-name[. = 'attr-name']">
	<xsl:attribute name="attr-name">name</xsl:attribute>
</xsl:template>

<xsl:template match="create-rule/variable-ref/@attr-name[. = 'class-name'] |
					match-class/variable-ref/@attr-name[. = 'class-name'] |
					create-rule/request-data/@attr-name[. = 'class-name'] |
					match-class/request-data/@attr-name[. = 'class-name']">
	<xsl:attribute name="attr-name">name</xsl:attribute>
</xsl:template>

<xsl:template match="match-path/variable-ref/@attr-name[. = 'prefix'] |
					match-path/request-data/@attr-name[. = 'prefix']"/>

<xsl:template match="match-path/request-data/@type | @type[. = 'match-path']">
	<xsl:attribute name="type">text</xsl:attribute>
</xsl:template>


<xsl:template match="request-data[@type='match-path']">
	<or>
		<if-src-dn op="in-subtree">
			<xsl:copy>
				<xsl:apply-templates select="@*|node()"/>
			</xsl:copy>
		</if-src-dn>
	</or>
	<or>
		<if-src-dn op="not-equal">
			<xsl:copy>
				<xsl:apply-templates select="@*|node()"/>
			</xsl:copy>
		</if-src-dn>
	</or>
</xsl:template>

<xsl:template match="node()|@*">
	<xsl:copy>
		<xsl:apply-templates select="@*|node()"/>
	</xsl:copy>
</xsl:template>

<!-- try to add a config-manifest and/or application-schema if there is none -->
<xsl:template match="driver-configuration/attributes">
	<xsl:copy>
		<xsl:apply-templates select="@*"/>
		<xsl:variable name="dnformat-rtf">
			<xxx>
				<xsl:call-template name="add-application-dn"/>
			</xxx>
		</xsl:variable>
		<xsl:variable name="dnformat" xmlns:exsl="http://exslt.org/common" select="exsl:node-set($dnformat-rtf)/xxx/@*"/>
		<xsl:if test="not(application-schema) and $dnformat">
			<application-schema>
				<schema-def>
					<xsl:copy-of select="$dnformat"/>
				</schema-def>
			</application-schema>
		</xsl:if>		
		<xsl:if test="not(config-manifest)">
			<config-manifest>
				<manifest/>
			</config-manifest>
		</xsl:if>		
		<xsl:apply-templates select="node()"/>
	</xsl:copy>
</xsl:template>

<!-- try to add a schema-def if there is none -->
<xsl:template match="driver-configuration/attributes/application-schema">
	<xsl:copy>
		<xsl:apply-templates select="@*"/>
		<xsl:variable name="dnformat-rtf">
			<xxx>
				<xsl:call-template name="add-application-dn"/>
			</xxx>
		</xsl:variable>
		<xsl:variable name="dnformat" xmlns:exsl="http://exslt.org/common" select="exsl:node-set($dnformat-rtf)/xxx/@*"/>
		<xsl:if test="not(schema-def) and $dnformat">
			<schema-def>
				<xsl:copy-of select="$dnformat"/>
			</schema-def>
		</xsl:if>		
		<xsl:apply-templates select="node()"/>
	</xsl:copy>
</xsl:template>


<!-- try to add a manifest entry for app-dn-format if there is none -->
<xsl:template match="driver-configuration/attributes/application-schema/schema-def">
	<xsl:copy>
		<xsl:if test="not(@dn-format | @dn-delims)">
			<xsl:call-template name="add-application-dn"/>
		</xsl:if>		
		<xsl:apply-templates select="@*|node()"/>
	</xsl:copy>
</xsl:template>

<!-- add a manifest entry for app-dn-format if it can be discerned from existing placement rules -->
<xsl:template name="add-application-dn">
	<xsl:variable name="format" select="//placement-rules/@src-dn-format[. != 'slash'] | //placement-rules/@src-dn-format/@dest-dn-format[. != 'slash']"/>
	<xsl:variable name="delims" select="//placement-rules/@src-dn-delims | //placement-rules/@dest-dn-delims"/>
	<xsl:choose>
		<xsl:when test="$delims">
			<xsl:attribute name="dn-delims">
				<xsl:value-of select="$delims"/>
			</xsl:attribute>
		</xsl:when>
		<xsl:when test="$format">
			<xsl:attribute name="dn-format">
				<xsl:value-of select="$format"/>
			</xsl:attribute>
		</xsl:when>
	</xsl:choose>
</xsl:template>


</xsl:transform>
