/******************************************************************
Copyright (c) 1996 Novell, Inc. All Rights Reserved.

THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND
TREATIES. USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE
LICENSE AGREEMENT ACCOMPANYING THE SOFTWARE DEVELOPMENT KIT (SDK)
THAT CONTAINS THIS WORK. PURSUANT TO THE SDK LICENSE AGREEMENT,
NOVELL HEREBY GRANTS TO DEVELOPER A ROYALTY-FREE, NON-EXCLUSIVE
LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS PRODUCT. NOVELL
GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF
DEVELOPER'S PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO
DEVELOPER OR DEVELOPER'S CUSTOMERS WITH RESPECT TO THIS CODE.
******************************************************************/

package com.novell.nds.dirxml.driver.jdbc.db.lock;





/**
 * Default Oracle lock generator implementation.
 * <p>
 * NOTE:  Oracle SELECT statements can't block even if the table is
 * exclusively locked.
 */
public
class OraLockGenerator
    extends DBLockGenerator
{

    public
    DBLock
    generateInsertLock()
    {
        DBLock lock;
        DBLockStatement stmt;

        //write operation against views be redefined; lock resources
        //  in instead-of-triggers
        if(isTable())
        {
            Lib.Clear(_buffer);

            lock = new DBLock();
            _buffer.append("LOCK TABLE ");
            _buffer.append(_tableName);
            _buffer.append(" IN ROW SHARE MODE NOWAIT");

            stmt = new DBLockStatement(_buffer.toString());
            lock.addLockStatement(stmt);
        }
        else
        {
            lock = EMPTY_LOCK;
        }

        return lock;
    }//generateInsertLock()


    public
    DBLock
    generateUpdateLock()
    {
        return generateLock();
    }


    public
    DBLock
    generateDeleteLock()
    {
        return generateLock();
    }


    private
    DBLock
    generateLock()
    {
        DBLock lock;
        DBLockStatement stmt;

        //write operation against views be redefined; lock resources
        //  in instead-of-triggers
        if(isTable())
        {
            Lib.Clear(_buffer);

            lock = new DBLock();
            if(_whereClause == null)
            {
                _buffer.append("LOCK TABLE ");
                _buffer.append(_tableName);
                _buffer.append(" IN ROW EXCLUSIVE MODE NOWAIT");

                stmt = new DBLockStatement(_buffer.toString());
            }
            else
            {
                _buffer.append("SELECT ROWID FROM ");
                _buffer.append(_tableName);
                _buffer.append(_whereClause);
                _buffer.append(" FOR UPDATE NOWAIT");

                stmt = new DBLockStatement(_buffer.toString());
            }

            lock.addLockStatement(stmt);
        }//if
        else
        {
            lock = EMPTY_LOCK;
        }

        return lock;
    }//generateLock()

}//class OracleLockGenerator
